<?php
//
// Custom Text Dashlet
// Copyright (c) 2025 Nagios Enterprises, LLC. All rights reserved.
//

include_once(dirname(__FILE__) . '/../dashlethelper.inc.php');

// Run the initialization function
custom_text_dashlet_init();

function custom_text_dashlet_init()
{
    $name = "custom_text";

    $args = array(
        DASHLET_NAME => $name,
        DASHLET_VERSION => "1.0.0",
        DASHLET_AUTHOR => "Nagios Enterprises, LLC",
        DASHLET_DESCRIPTION => _("Display custom text in a dashlet with full styling options."),
        DASHLET_COPYRIGHT => "Dashlet Copyright &copy; 2025 Nagios Enterprises.",
        DASHLET_LICENSE => "MIT",
        DASHLET_HOMEPAGE => "https://www.nagios.com",
        DASHLET_FUNCTION => "custom_text_dashlet_func",
        DASHLET_TITLE => _("Custom Text"),
        DASHLET_OUTBOARD_CLASS => "custom_text_outboardclass",
        DASHLET_INBOARD_CLASS => "custom_text_inboardclass",
        DASHLET_PREVIEW_CLASS => "custom_text_previewclass",
        DASHLET_WIDTH => "300",
        DASHLET_HEIGHT => "200",
        DASHLET_OPACITY => "1.0",
        DASHLET_BACKGROUND => "",
    );

    register_dashlet($name, $args);
}

function custom_text_dashlet_func($mode = DASHLET_MODE_PREVIEW, $id = "", $args = null)
{
    $output = "";
    $imgbase = get_dashlet_url_base("custom_text") . "/images/";

    switch ($mode) {
        case DASHLET_MODE_GETCONFIGHTML:
            if(get_theme() == "xi5") {
                $input_background = "white;";
            } else if (get_theme() == "xi5dark") {
                $input_background = "#333;";
            }
            else if (get_theme() == "neptunelight" || get_theme() == "neptunelightdark") {
                $input_background = "var(--input);";
            }
            else{
                $input_background = "white;";
            }
            $output = '
            <div class="form-group">
                <label for="displaytext">'._('Text to Display').'</label>
                <div>
                    <textarea name="displaytext" class="form-control" id="displaytext" rows="4"></textarea>
                </div>
            </div>
            
            <div class="form-group">
                <label for="fontfamily">'._('Font Family').'</label>
                <div class="font-select-container">
                    <input type="text" class="font-search" placeholder="Search fonts..." 
                           oninput="filterFonts(this.value)" onclick="showFontDropdown()">
                    <div class="font-dropdown" id="fontDropdown">
                        <select name="fontfamily" class="form-control" id="fontfamily" size="15" 
                                style="height: 250px;" onchange="updateSelectedFont(this)">
                            <!-- Sans-Serif Fonts -->
                            <option value="Arial, sans-serif" style="font-family: Arial">Arial</option>
                            <option value="Arial Black, sans-serif" style="font-family: Arial Black">Arial Black</option>
                            <option value="Avenir, sans-serif" style="font-family: Avenir">Avenir</option>
                            <option value="Calibri, sans-serif" style="font-family: Calibri">Calibri</option>
                            <option value="Candara, sans-serif" style="font-family: Candara">Candara</option>
                            <option value="Century Gothic, sans-serif" style="font-family: Century Gothic">Century Gothic</option>
                            <option value="Corbel, sans-serif" style="font-family: Corbel">Corbel</option>
                            <option value="Franklin Gothic Medium, sans-serif" style="font-family: Franklin Gothic Medium">Franklin Gothic</option>
                            <option value="Futura, sans-serif" style="font-family: Futura">Futura</option>
                            <option value="Geneva, sans-serif" style="font-family: Geneva">Geneva</option>
                            <option value="Gill Sans, sans-serif" style="font-family: Gill Sans">Gill Sans</option>
                            <option value="Helvetica, sans-serif" style="font-family: Helvetica">Helvetica</option>
                            <option value="Helvetica Neue, sans-serif" style="font-family: Helvetica Neue">Helvetica Neue</option>
                            <option value="Impact, sans-serif" style="font-family: Impact">Impact</option>
                            <option value="Lucida Grande, sans-serif" style="font-family: Lucida Grande">Lucida Grande</option>
                            <option value="Lucida Sans, sans-serif" style="font-family: Lucida Sans">Lucida Sans</option>
                            <option value="Optima, sans-serif" style="font-family: Optima">Optima</option>
                            <option value="Segoe UI, sans-serif" style="font-family: Segoe UI">Segoe UI</option>
                            <option value="Tahoma, sans-serif" style="font-family: Tahoma">Tahoma</option>
                            <option value="Trebuchet MS, sans-serif" style="font-family: Trebuchet MS">Trebuchet MS</option>
                            <option value="Verdana, sans-serif" style="font-family: Verdana">Verdana</option>

                            <!-- Serif Fonts -->
                            <option value="Baskerville, serif" style="font-family: Baskerville">Baskerville</option>
                            <option value="Big Caslon, serif" style="font-family: Big Caslon">Big Caslon</option>
                            <option value="Bodoni MT, serif" style="font-family: Bodoni MT">Bodoni MT</option>
                            <option value="Book Antiqua, serif" style="font-family: Book Antiqua">Book Antiqua</option>
                            <option value="Calisto MT, serif" style="font-family: Calisto MT">Calisto MT</option>
                            <option value="Cambria, serif" style="font-family: Cambria">Cambria</option>
                            <option value="Didot, serif" style="font-family: Didot">Didot</option>
                            <option value="Garamond, serif" style="font-family: Garamond">Garamond</option>
                            <option value="Georgia, serif" style="font-family: Georgia">Georgia</option>
                            <option value="Goudy Old Style, serif" style="font-family: Goudy Old Style">Goudy Old Style</option>
                            <option value="Hoefler Text, serif" style="font-family: Hoefler Text">Hoefler Text</option>
                            <option value="Lucida Bright, serif" style="font-family: Lucida Bright">Lucida Bright</option>
                            <option value="Palatino, serif" style="font-family: Palatino">Palatino</option>
                            <option value="Perpetua, serif" style="font-family: Perpetua">Perpetua</option>
                            <option value="Rockwell, serif" style="font-family: Rockwell">Rockwell</option>
                            <option value="Times New Roman, serif" style="font-family: Times New Roman">Times New Roman</option>

                            <!-- Monospace Fonts -->
                            <option value="Andale Mono, monospace" style="font-family: Andale Mono">Andale Mono</option>
                            <option value="Consolas, monospace" style="font-family: Consolas">Consolas</option>
                            <option value="Courier New, monospace" style="font-family: Courier New">Courier New</option>
                            <option value="Lucida Console, monospace" style="font-family: Lucida Console">Lucida Console</option>
                            <option value="Monaco, monospace" style="font-family: Monaco">Monaco</option>

                            <!-- Display/Decorative Fonts -->
                            <option value="Brush Script MT, cursive" style="font-family: Brush Script MT">Brush Script MT</option>
                            <option value="Comic Sans MS, cursive" style="font-family: Comic Sans MS">Comic Sans MS</option>
                            <option value="Copperplate, fantasy" style="font-family: Copperplate">Copperplate</option>
                            <option value="Papyrus, fantasy" style="font-family: Papyrus">Papyrus</option>
                            <option value="Luminari, fantasy" style="font-family: Luminari">Luminari</option>
                            <option value="Party LET, fantasy" style="font-family: Party LET">Party LET</option>
                            <option value="Snell Roundhand, cursive" style="font-family: Snell Roundhand">Snell Roundhand</option>
                        </select>
                    </div>
                </div>
            </div>

            <div class="form-group">
                <label for="fontweight">'._('Font Weight').'</label>
                <div>
                    <select name="fontweight" class="form-control" id="fontweight" onchange="toggleCustomWeight()">
                        <option value="normal">Normal</option>
                        <option value="bold">Bold</option>
                        <option value="lighter">Light</option>
                        <option value="custom">Custom</option>
                    </select>
                    <div id="customweightcontainer" style="display: none;">
                        <input type="range" name="customfontweight" class="form-control" id="customfontweight" 
                               min="100" max="900" value="400" step="5">
                        <span id="fontweightvalue">400</span>
                    </div>
                </div>
            </div>


            <div class="form-group">
                <label>'._('Italic').'</label>
                <div>
                    <input type="checkbox" name="italic" id="italic">
                </div>
            </div>

            <div class="form-group">
                <label>'._('Underline').'</label>
                <div>
                    <input type="checkbox" name="underline" id="underline">
                </div>
            </div>

            <div class="form-group">
                <label>'._('Strikethrough').'</label>
                <div>
                    <input type="checkbox" name="strikethrough" id="strikethrough">
                </div>
            </div>


            <div class="form-group">
                <label for="textcolor">'._('Text Color').'</label>
                <div>
                    <select name="textcolor" class="form-control" id="textcolor">
                        <option value="white">White</option>
                        <option value="black">Black</option>
                        <option value="custom">Custom Color</option>
                    </select>
                    <input type="color" name="customtextcolor" class="form-control" id="customtextcolor" 
                           value="#000000" style="margin-top: 5px; display: none; width: 100px; padding: 0;">
                </div>
            </div>

            <div class="form-group">
                <label for="textopacity">'._('Text Opacity').'</label>
                <div>
                    <input type="range" name="textopacity" class="form-control" id="textopacity" 
                           min="0" max="100" value="100" step="1">
                    <span id="textopacityvalue">100%</span>
                </div>
            </div>

            <div class="form-group">
                <label for="bgcolor">'._('Background Color').'</label>
                <div>
                    <select name="bgcolor" class="form-control" id="bgcolor">
                        <option value="transparent">Transparent</option>
                        <option value="white">White</option>
                        <option value="black">Black</option>
                        <option value="custom">Custom Color</option>
                    </select>
                    <input type="color" name="custombgcolor" class="form-control" id="custombgcolor" 
                           value="#ffffff" style="margin-top: 5px; display: none; width: 100px; padding: 0;">
                </div>
            </div>

            <div class="form-group">
                <label for="bgopacity">'._('Background Opacity').'</label>
                <div>
                    <input type="range" name="bgopacity" class="form-control" id="bgopacity" 
                           min="0" max="100" value="100" step="1">
                    <span id="bgopacityvalue">100%</span>
                </div>
            </div>

            <div class="form-group">
                <label for="bordercolor">'._('Border Color').'</label>
                <div>
                    <select name="bordercolor" class="form-control" id="bordercolor">
                        <option value="transparent">Transparent</option>
                        <option value="white">White</option>
                        <option value="black">Black</option>
                        <option value="custom">Custom Color</option>
                    </select>
                    <input type="color" name="custombordercolor" class="form-control" id="custombordercolor" 
                           value="#000000" style="margin-top: 5px; display: none; width: 100px; padding: 0;">
                </div>
            </div>

            <div class="form-group">
                <label for="borderwidth">'._('Border Width').'</label>
                <div>
                    <input type="range" name="borderwidth" class="form-control" id="borderwidth" 
                           min="0" max="50" value="1" step="1">
                    <span id="borderwidthvalue">1px</span>
                </div>
            </div>

            <div class="form-group">
                <label for="borderradius">'._('Border Radius').'</label>
                <div>
                    <input type="range" name="borderradius" class="form-control" id="borderradius" 
                           min="0" max="360" value="0" step="1">
                    <span id="borderradiusvalue">0px</span>
                </div>
            </div>

            <div class="form-group">
                <label for="borderopacity">'._('Border Opacity').'</label>
                <div>
                    <input type="range" name="borderopacity" class="form-control" id="borderopacity" 
                           min="0" max="100" value="100" step="1">
                    <span id="borderopacityvalue">100%</span>
                </div>
            </div>

            <div class="form-group">
                <label>'._('Preview').'</label>
                <div id="dashletPreview" class="dashlet-preview" style="
                    background-color: var(--input);
                    padding: 10px;
                    height: 200px;
                    width: 400px;
                    margin-top: 10px;
                    display: flex;
                    align-items: center;
                    justify-content: center;
                    text-align: center;
                    overflow: hidden;
                    position: relative;
                    box-sizing: border-box;
                ">
                    <div id="previewContent" style="
                        max-width: 100%;
                        max-height: 100%;
                        word-wrap: break-word;
                    "></div>
                </div>
            </div>

            <script>
                // Show/hide color pickers
                document.getElementById("textcolor").onchange = function() {
                    document.getElementById("customtextcolor").style.display = 
                        this.value === "custom" ? "block" : "none";
                };
                document.getElementById("bgcolor").onchange = function() {
                    document.getElementById("custombgcolor").style.display = 
                        this.value === "custom" ? "block" : "none";
                };
                document.getElementById("bordercolor").onchange = function() {
                    document.getElementById("custombordercolor").style.display = 
                        this.value === "custom" ? "block" : "none";
                };

                // Update opacity and width value displays
                document.getElementById("textopacity").oninput = function() {
                    document.getElementById("textopacityvalue").textContent = this.value + "%";
                };
                document.getElementById("bgopacity").oninput = function() {
                    document.getElementById("bgopacityvalue").textContent = this.value + "%";
                };
                document.getElementById("borderopacity").oninput = function() {
                    document.getElementById("borderopacityvalue").textContent = this.value + "%";
                };
                document.getElementById("borderwidth").oninput = function() {
                    document.getElementById("borderwidthvalue").textContent = this.value + "px";
                };
                document.getElementById("borderradius").oninput = function() {
                    document.getElementById("borderradiusvalue").textContent = this.value + "px";
                };

                function updateSelectedFont(select) {
                    var selectedOption = select.options[select.selectedIndex];
                    var searchInput = select.parentElement.previousElementSibling;
                    searchInput.value = selectedOption.text;
                    searchInput.style.fontFamily = selectedOption.value;
                    document.getElementById("fontDropdown").style.display = "none";
                }

                function showFontDropdown() {
                    var select = document.getElementById("fontfamily");
                    var searchInput = select.parentElement.previousElementSibling;
                    // Set the initial font if it exists
                    if (select.selectedIndex >= 0) {
                        searchInput.style.fontFamily = select.options[select.selectedIndex].value;
                    }
                    document.getElementById("fontDropdown").style.display = "block";
                }

                function filterFonts(query) {
                    query = query.toLowerCase();
                    var select = document.getElementById("fontfamily");
                    var options = select.options;
                    
                    for (var i = 0; i < options.length; i++) {
                        var text = options[i].text.toLowerCase();
                        options[i].style.display = text.includes(query) ? "" : "none";
                    }
                }

                // Close dropdown when clicking outside
                document.addEventListener("click", function(e) {
                    var container = document.querySelector(".font-select-container");
                    if (!container.contains(e.target)) {
                        document.getElementById("fontDropdown").style.display = "none";
                    }
                });

                function toggleCustomWeight() {
                    var weightSelect = document.getElementById("fontweight");
                    var customContainer = document.getElementById("customweightcontainer");
                    customContainer.style.display = weightSelect.value === "custom" ? "block" : "none";
                }

                document.getElementById("customfontweight").oninput = function() {
                    document.getElementById("fontweightvalue").textContent = this.value;
                };

                function updatePreview() {
                    var preview = document.getElementById("dashletPreview");
                    var content = document.getElementById("previewContent");
                    if (!preview || !content) return;

                    var text = document.getElementById("displaytext").value;
                    content.innerHTML = text ? text.replace(/\n/g, "<br>") : "";
                    
                    // Apply font settings
                    content.style.fontFamily = document.getElementById("fontfamily").value;
                    var weightSelect = document.getElementById("fontweight");
                    if (weightSelect.value === "custom") {
                        content.style.fontWeight = document.getElementById("customfontweight").value;
                    } else {
                        content.style.fontWeight = weightSelect.value;
                    }
                    
                    // Update text color and opacity
                    var textColor = document.getElementById("textcolor").value;
                    if (textColor === "custom") {
                        textColor = document.getElementById("customtextcolor").value;
                    }
                    if (textColor === "white") textColor = "#FFFFFF";
                    if (textColor === "black") textColor = "#000000";

                    // Apply italic style
                    var italicCheckbox = document.getElementById("italic");
                    if (italicCheckbox.checked) {
                        content.style.fontStyle = "italic";
                    } else {
                        content.style.fontStyle = "normal";
                    }

                    var underlineCheckbox = document.getElementById("underline");
                    var strikethroughCheckbox = document.getElementById("strikethrough");

                    // Combine text decoration styles
                    var textDecoration = [];
                    if (underlineCheckbox.checked) {
                        textDecoration.push("underline");
                    }
                    if (strikethroughCheckbox.checked) {
                        textDecoration.push("line-through");
                    }

                    content.style.textDecoration = textDecoration.join(" ");
                    
                    // Convert text color to rgba
                    var textRgb = textColor.match(/^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i);
                    if (textRgb) {
                        var r = parseInt(textRgb[1], 16);
                        var g = parseInt(textRgb[2], 16);
                        var b = parseInt(textRgb[3], 16);
                        var textOpacity = document.getElementById("textopacity").value / 100;
                        content.style.color = `rgba(${r},${g},${b},${textOpacity})`;
                    }
                    content.style.opacity = 1; // Reset content opacity
                    
                    // Update background color and opacity
                    var bgColor = document.getElementById("bgcolor").value;
                    if (bgColor === "custom") {
                        bgColor = document.getElementById("custombgcolor").value;
                    }
                    if (bgColor === "white") bgColor = "#FFFFFF";
                    if (bgColor === "black") bgColor = "#000000";
                    
                    if (bgColor !== "transparent") {
                        var bgRgb = bgColor.match(/^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i);
                        if (bgRgb) {
                            var r = parseInt(bgRgb[1], 16);
                            var g = parseInt(bgRgb[2], 16);
                            var b = parseInt(bgRgb[3], 16);
                            var bgOpacity = document.getElementById("bgopacity").value / 100;
                            preview.style.backgroundColor = `rgba(${r},${g},${b},${bgOpacity})`;
                        }
                    } else {
                        preview.style.backgroundColor = "transparent";
                    }
                    preview.style.opacity = 1; // Reset preview opacity
                    
                    // Update border color, width, and opacity
                    var borderColor = document.getElementById("bordercolor").value;
                    var borderWidth = document.getElementById("borderwidth").value;
                    if (borderColor === "custom") {
                        borderColor = document.getElementById("custombordercolor").value;
                    }
                    if (borderColor === "white") borderColor = "#FFFFFF";
                    if (borderColor === "black") borderColor = "#000000";
                    
                    if (borderColor !== "transparent") {
                        var borderRgb = borderColor.match(/^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i);
                        if (borderRgb) {
                            var r = parseInt(borderRgb[1], 16);
                            var g = parseInt(borderRgb[2], 16);
                            var b = parseInt(borderRgb[3], 16);
                            var borderOpacity = document.getElementById("borderopacity").value / 100;
                            preview.style.border = `${borderWidth}px solid rgba(${r},${g},${b},${borderOpacity})`;
                        }
                    } else {
                        preview.style.border = `${borderWidth}px solid transparent`;
                    }
                    
                    // Update border radius
                    var borderRadius = document.getElementById("borderradius").value;
                    preview.style.borderRadius = borderRadius + "px";
                    
                    // Update text scaling
                    var container = preview.getBoundingClientRect();
                    var fontSize = 100;
                    content.style.fontSize = fontSize + "px";
                    
                    // Binary search for the best font size
                    var minSize = 1;
                    var maxSize = 1000;
                    
                    while (minSize <= maxSize) {
                        fontSize = Math.floor((minSize + maxSize) / 2);
                        content.style.fontSize = fontSize + "px";
                        
                        if (content.scrollHeight <= container.height - 20 && 
                            content.scrollWidth <= container.width - 20) {
                            minSize = fontSize + 1;
                        } else {
                            maxSize = fontSize - 1;
                        }
                    }
                    
                    // Set final font size
                    fontSize = Math.max(1, maxSize);
                    content.style.fontSize = fontSize + "px";
                }

                // Add event listeners for all inputs that affect the preview
                document.getElementById("displaytext").addEventListener("input", updatePreview);
                document.getElementById("fontfamily").addEventListener("change", updatePreview);
                document.getElementById("fontweight").addEventListener("change", updatePreview);
                document.getElementById("italic").addEventListener("change", updatePreview);
                document.getElementById("underline").addEventListener("change", updatePreview);
                document.getElementById("strikethrough").addEventListener("change", updatePreview);
                document.getElementById("textcolor").addEventListener("change", updatePreview);
                document.getElementById("customtextcolor").addEventListener("input", updatePreview);
                document.getElementById("textopacity").addEventListener("input", updatePreview);
                document.getElementById("bgcolor").addEventListener("change", updatePreview);
                document.getElementById("custombgcolor").addEventListener("input", updatePreview);
                document.getElementById("bgopacity").addEventListener("input", updatePreview);
                document.getElementById("bordercolor").addEventListener("change", updatePreview);
                document.getElementById("custombordercolor").addEventListener("input", updatePreview);
                document.getElementById("borderwidth").addEventListener("input", updatePreview);
                document.getElementById("borderradius").addEventListener("input", updatePreview);
                document.getElementById("borderopacity").addEventListener("input", updatePreview);
                document.getElementById("customfontweight").addEventListener("input", updatePreview);

                // Initial preview
                updatePreview();
            </script>

            <style>
                .font-select-container {
                    position: relative;
                }
                .font-search {
                    width: 100%;
                    padding: 6px 12px;
                    border: 1px solid var(--border);
                    border-radius: var(--radius);
                    height: 34px;
                    font-size: 14px;
                    line-height: 1.42857143;
                    background-color: var(--input);
                    color: var(--foreground);
                    font-family: inherit;
                }
                .font-dropdown {
                    display: none;
                    position: absolute;
                    width: 100%;
                    z-index: 1000;
                    background-color: '.$input_background.';
                    border: 1px solid var(--border);
                    border-radius: var(--radius);
                    margin-top: 2px;
                    box-shadow: 0 2px 5px rgba(0,0,0,0.2);
                }
                .font-dropdown select {
                    width: 100%;
                    border: none;
                    outline: none;
                    padding: 0;
                    background-color: var(--input);
                    color: var(--foreground);
                }
                .font-dropdown select option {
                    padding: 8px 12px;
                    cursor: pointer;
                    font-size: 14px;
                    line-height: 1.2;
                    height: 35px;
                    background-color: var(--input);
                    color: var(--foreground);
                }
                .font-dropdown select option:hover,
                .font-dropdown select option:focus {
                    background-color: var(--secondary);
                }

                /* Make the dropdown scrollbar match theme */
                .font-dropdown select::-webkit-scrollbar {
                    width: 12px;
                }
                .font-dropdown select::-webkit-scrollbar-track {
                    background-color: var(--input);
                    border-radius: var(--radius);
                }
                .font-dropdown select::-webkit-scrollbar-thumb {
                    background-color: var(--muted);
                    border-radius: var(--radius);
                    border: 2px solid transparent;
                    background-clip: padding-box;
                }
                .font-dropdown select::-webkit-scrollbar-thumb:hover {
                    background-color: var(--secondary);
                }

                /* Ensure text input matches form-control styling */
                .font-search.form-control {
                    display: block;
                    width: 100%;
                    height: 34px;
                    padding: 6px 12px;
                    font-size: 14px;
                    line-height: 1.42857143;
                    background-color: var(--input);
                    color: var(--foreground);
                    border: 1px solid var(--border);
                    border-radius: var(--radius);
                }
            </style>
            ';
            break;

        case DASHLET_MODE_PREVIEW:
            if(!is_neptune()) {
                $output = "<p><img src='" . $imgbase . "preview_other.png'></p>";
            } else if (get_theme() == "neptunelight") {
                $output = "<img src='" . $imgbase . "preview.png' style='max-width: 100%; height: auto;'>";
            } else {
                $output = "<img src='" . $imgbase . "text_preview.png' style='max-width: 100%'>";
            }
            break;

        case DASHLET_MODE_OUTBOARD:
        case DASHLET_MODE_INBOARD:
    
            $displaytext = isset($args["displaytext"]) ? htmlspecialchars($args["displaytext"]) : "";
            $fontfamily = isset($args["fontfamily"]) ? $args["fontfamily"] : "Arial, sans-serif";
            $fontweight = isset($args["fontweight"]) ? $args["fontweight"] : "normal";
            
            // Process font weight
            if ($fontweight === "custom") {
                $fontweight = isset($args["customfontweight"]) ? $args["customfontweight"] : "400";
            }



            // Process text color
            $textColor = "black";
            if (isset($args["textcolor"])) {
                if ($args["textcolor"] === "custom" && isset($args["customtextcolor"])) {
                    $opacity = isset($args["textopacity"]) ? $args["textopacity"] / 100 : 1;
                    $textColor = "rgba(" . implode(",", sscanf($args["customtextcolor"], "#%02x%02x%02x")) . ",$opacity)";
                } else if ($args["textcolor"] === "white") {
                    $opacity = isset($args["textopacity"]) ? $args["textopacity"] / 100 : 1;
                    $textColor = "rgba(255,255,255,$opacity)";
                } else if ($args["textcolor"] === "black") {
                    $opacity = isset($args["textopacity"]) ? $args["textopacity"] / 100 : 1;
                    $textColor = "rgba(0,0,0,$opacity)";
                }
            }

            // Process italic
            $italic = false;
            if (isset($args["italic"])) {
                $italic = $args["italic"] === "on"; // Check for "true" or "1"
            }

            $italic = $italic ? "italic" : "normal";


            // Process underline
            $underline = isset($args["underline"]) && $args["underline"] === "on" ? "underline" : "none";

            // Process strikethrough
            $strikethrough = isset($args["strikethrough"]) && $args["strikethrough"] === "on" ? "line-through" : "none";

            // Combine text decoration styles
            $textDecoration = [];
            if ($underline === "underline") {
                $textDecoration[] = "underline";
            }
            if ($strikethrough === "line-through") {
                $textDecoration[] = "line-through";
            }

            // Set the final text decoration style
            $finalTextDecoration = implode(" ", $textDecoration) ?: "none";

            // Process background color
            $bgColor = "transparent";
            if (isset($args["bgcolor"])) {
                if ($args["bgcolor"] === "custom" && isset($args["custombgcolor"])) {
                    $opacity = isset($args["bgopacity"]) ? $args["bgopacity"] / 100 : 1;
                    $bgColor = "rgba(" . implode(",", sscanf($args["custombgcolor"], "#%02x%02x%02x")) . ",$opacity)";
                } else if ($args["bgcolor"] === "white") {
                    $opacity = isset($args["bgopacity"]) ? $args["bgopacity"] / 100 : 1;
                    $bgColor = "rgba(255,255,255,$opacity)";
                } else if ($args["bgcolor"] === "black") {
                    $opacity = isset($args["bgopacity"]) ? $args["bgopacity"] / 100 : 1;
                    $bgColor = "rgba(0,0,0,$opacity)";
                }
            }

            // Process border
            $borderWidth = isset($args["borderwidth"]) ? intval($args["borderwidth"]) : 0;
            $borderColor = "transparent";
            if (isset($args["bordercolor"])) {
                if ($args["bordercolor"] === "custom" && isset($args["custombordercolor"])) {
                    $opacity = isset($args["borderopacity"]) ? $args["borderopacity"] / 100 : 1;
                    $borderColor = "rgba(" . implode(",", sscanf($args["custombordercolor"], "#%02x%02x%02x")) . ",$opacity)";
                } else if ($args["bordercolor"] === "white") {
                    $opacity = isset($args["borderopacity"]) ? $args["borderopacity"] / 100 : 1;
                    $borderColor = "rgba(255,255,255,$opacity)";
                } else if ($args["bordercolor"] === "black") {
                    $opacity = isset($args["borderopacity"]) ? $args["borderopacity"] / 100 : 1;
                    $borderColor = "rgba(0,0,0,$opacity)";
                }
            }
            
            // Handle border radius
            $borderRadius = isset($args["borderradius"]) ? intval($args["borderradius"]) : 0;
            
            $output = '
            <script>
                document.addEventListener("DOMContentLoaded", function() {
                    setTimeout(function() {
                        var element = document.getElementById("text-container-'.$id.'");
                        while (element && !element.classList.contains("dashboarddashletcontainer")) {
                            element = element.parentElement;
                        }
                        
                        if (element) {
                            element.style.setProperty("background-color", "transparent", "important");
                            element.style.setProperty("border", "0px solid var(--border)", "important");
                        }
                    }, 100);
                });
            </script>
            <div class="text-dashlet-wrapper" style="position: relative; height: 100%; width: 100%;">
                <div class="text-display-container" id="text-container-'.$id.'" style="
                    padding: 10px;
                    font-family: '.$fontfamily.';
                    font-weight: '.$fontweight.';
                    font-style: '.$italic.';
                    text-decoration: '.$finalTextDecoration.';
                    color: '.$textColor.';
                    background-color: '.$bgColor.';
                    border: '.$borderWidth.'px solid '.$borderColor.';
                    border-radius: '.$borderRadius.'px;
                    height: 100%;
                    width: 100%;
                    box-sizing: border-box;
                    display: flex;
                    align-items: center;
                    justify-content: center;
                    text-align: center;
                    overflow: hidden;
                    position: relative;
                ">
                    <div id="text-content-'.$id.'" style="
                        text-align: center;
                        word-wrap: break-word;
                        max-width: 100%;
                    ">
                        '.nl2br($displaytext).'
                    </div>
                </div>
            </div>

            <script>

            (function() {
                function initResize() {
                    var container = document.getElementById("text-container-'.$id.'");
                    var content = document.getElementById("text-content-'.$id.'");
                    
                    if (!container || !content) return;
                    
                    function resizeText() {
                        var containerRect = container.getBoundingClientRect();
                        var fontSize = 100;
                        content.style.fontSize = fontSize + "px";
                        
                        // Binary search for the best font size
                        var minSize = 1;
                        var maxSize = 1000;
                        
                        while (minSize <= maxSize) {
                            fontSize = Math.floor((minSize + maxSize) / 2);
                            content.style.fontSize = fontSize + "px";
                            
                            if (content.scrollHeight <= containerRect.height - 20 && 
                                content.scrollWidth <= containerRect.width - 20) {
                                minSize = fontSize + 1;
                            } else {
                                maxSize = fontSize - 1;
                            }
                        }
                        
                        // Set final font size
                        fontSize = Math.max(1, maxSize);
                        content.style.fontSize = fontSize + "px";
                    }
                    
                    resizeText();
                    window.addEventListener("resize", resizeText);
                    if (window.ResizeObserver) {
                        new ResizeObserver(resizeText).observe(container);
                    }
                }
                
                setTimeout(initResize, 100);
            })();
            </script>
            ';
            break;
    }

    return $output;
}

